/**
 * Repraesentation einen von einem Menschen gesteuerten Kalah-spielenden Hamster
 * 
 * @author Dietrich Boles (Universitaet Oldenburg)
 * @version 1.0 (29.01.2006)
 * 
 */
public class MenschHamster extends AllroundHamster implements Spieler {

    /**
     * true bedeutet, dass es sich um Spieler A handelt; false bedeutet, dass es
     * sich um Spieler A handelt;
     */
    protected boolean istSpielerA;

    /**
     * die zugrunde liegenden Spielregeln
     */
    protected Spielregeln regeln;

    /**
     * initialisiert einen neuen SpielHamster
     * 
     */
    public MenschHamster() {
        super(0, 0, Hamster.OST, 0);
        this.istSpielerA = false;
        this.regeln = null;
    }

    /**
     * kennzeichnet einen Spieler als Spieler A oder B
     * 
     * @param istSpielerA
     *            true, falls es sich um Spieler A handeln soll; false, falls es
     *            sich um Spieler B handeln soll
     */
    public void setzeSpieler(boolean istSpielerA) {
        this.istSpielerA = istSpielerA;
    }

    /**
     * uebergibt dem Spieler die Spielregeln
     * 
     * @param regeln
     *            die gueltigen Spielregeln
     */
    public void teileSpielregelnMit(Spielregeln regeln) {
        this.regeln = regeln;
    }

    /**
     * in Ausgangsposition begeben
     */
    public void begibDichAufAusgangsKachel() {
        if (!this.maulLeer()) {
            
            // evtl. noch im Maul vorhandene Koerner ablegen
            this.laufeZuKachel(0, 0);
            this.gibAlle(); 
        }
        if (this.istSpielerA) {
            this.laufeZuKachel(0, Spielbrett.ANZAHL_MULDEN_PRO_SPIELER + 1);
            this.setzeBlickrichtung(Hamster.WEST);
        } else {
            this.laufeZuKachel(2, 0);
            this.setzeBlickrichtung(Hamster.OST);
        }     
    }

    /**
     * Ueberpruefung, welcher Spieler es ist
     * 
     * @return true, falls es sich um Spieler A handelt; false sonst
     */
    public boolean istSpielerA() {
        return this.istSpielerA;
    }

    /**
     * liefert A oder B, je nachdem, welcher Spielertyp es ist
     * 
     * @return "A", falls es sich um Spieler A handelt, "B" sonst
     */
    public String liefereSpielerTyp() {
        if (this.istSpielerA) {
            return "A";
        } else {
            return "B";
        }
    }

    /**
     * ermittelt den naechsten Spielzug, fuehrt ihn aus und liefert ihn
     * 
     * @return der naechste Spielzug des Spielers
     */
    public Spielzug naechsterSpielzug() {

        // naechsten Spielzug beim Benutzer erfragen
        int mulde = this.liesZahl("Spieler " + this.liefereSpielerTyp()
                + ": Welche Mulde?");
        Spielzug zug = new Spielzug(mulde);
        boolean ok = this.regeln.istSpielzugOk(this.istSpielerA, zug);
        while (!ok) {
            mulde = this.liesZahl("Fehler!\n Spieler "
                    + this.liefereSpielerTyp()
                    + ": Bitte korrekte Mulde angeben!");
            zug = new Spielzug(mulde);
            ok = this.regeln.istSpielzugOk(this.istSpielerA, zug);
        }

        // anschliessend wird der Spielzug im Hamster-Territorium
        // ausgefuehrt
        this.fuehreSpielzugAus(zug);

        // der ausgefuehrte Spielzug wird geliefert
        return zug;
    }

    /**
     * sammelt uebrig gebliebene Koerner ein und packt sie ins eigene Kalah
     */
    public void sammleResteUndBringSieZumKalah() {
        this.vor();
        int anzahl = 0;
        for (int i = 0; i < Spielbrett.ANZAHL_MULDEN_PRO_SPIELER; i++) {
            anzahl = anzahl + this.nimmAlle();
            this.vor();
        }
        this.linksUm();
        this.vor();
        this.gibAlle();
    }

    /**
     * beliebige Aktion, die ein Spieler nach Ende des Spiels durchfuehrt;
     * uebergeben wird die Information, ob der Spieler gewonnen hat; der Hamster
     * frisst zur Belohung alle Koerner seiner Kalah (auch wenn er verloren
     * hat)!
     * 
     * @param istSieger
     *            true, falls der Spieler gewonnen hat; sonst false
     */
    public void spielBeenden(boolean istSieger) {
        // sitzt bereits im Kalah
        this.nimmAlle(); // auch wenn er nicht gewonnen hat
    }

    // interne Methoden

    /**
     * der uebergebene Spielzug wird im Hamster-Territorium ausgefuehrt
     * 
     * @param zug
     *            der Spielzug, der ausgefuehrt werden soll
     */
    protected void fuehreSpielzugAus(Spielzug zug) {
        this.gotoMulde(zug.liefereMulde());
        int anzahl = this.nimmAlle();
        this.verteileKoerner(anzahl, zug.liefereMulde());
        this.begibDichAufAusgangsKachel();
    }

    /**
     * laeuft zur angegebenen Mulde
     * 
     * @param muldenNummer
     *            die Nummer der Mulde, zu der zu laufen ist
     */
    protected void gotoMulde(int muldenNummer) {
        if (this.istSpielerA) {
            this.vor(muldenNummer + 1);
        } else {
            this.vor(muldenNummer - Spielbrett.ANZAHL_MULDEN_PRO_SPIELER);
        }
    }

    /**
     * verteilt die aufgenommenen Koerner gemaess der Kalah-Spielregeln
     * 
     * @param anzahl
     *            die Anzahl der aufgenommenen Koerner
     * @param aktuelleMulde
     *            die im aktuellen Spielzug genannte Mulde
     */
    protected void verteileKoerner(int anzahl, int aktuelleMulde) {
        // zunaechst entgegen dem Uhrzeigersinn jeweils ein Korn
        // pro Mulde ablegen; Kalah des Gegners auslassen
        while (anzahl > 0) {
            if (aktuelleMulde == Spielbrett.KALAH_A - 1
                    || aktuelleMulde == Spielbrett.KALAH_B - 1) {
                this.vor();
                this.linksUm();
                this.vor();
                if (this.istSpielerA && aktuelleMulde == Spielbrett.KALAH_A - 1
                        || !this.istSpielerA
                        && aktuelleMulde == Spielbrett.KALAH_B - 1) {
                    this.gib();
                    anzahl = anzahl - 1;
                }
            } else if (aktuelleMulde == Spielbrett.KALAH_A
                    || aktuelleMulde == Spielbrett.KALAH_B) {
                this.vor();
                this.linksUm();
                this.vor();
                this.gib();
                anzahl = anzahl - 1;
            } else {
                this.vor();
                this.gib();
                anzahl = anzahl - 1;
            }
            aktuelleMulde = (aktuelleMulde + 1) % Spielbrett.ANZAHL_MULDEN;
        }

        // nun ueberpruefen, ob evtl. einige Koerner ins eigene Kalah
        // abgelegt werden duerfen
        if (this.istSpielerA
                && 0 <= aktuelleMulde
                && aktuelleMulde < Spielbrett.KALAH_A
                && Territorium.getAnzahlKoerner(this.getReihe(), this
                        .getSpalte()) == 1
                && Territorium.getAnzahlKoerner(this.getReihe() + 2, this
                        .getSpalte()) > 0) {
            this.nimm(); // das eine Korn der aktuellen Mulde fressen
            this.linksUm();
            this.vor();
            this.vor();
            this.nimmAlle(); // gegenueberliegende Mulde leer fressen
            this.gotoKalahA();
            this.gibAlle(); // in eigenes Kalah ablegen
        } else if (!this.istSpielerA
                && Spielbrett.KALAH_A < aktuelleMulde
                && aktuelleMulde < Spielbrett.KALAH_B
                && Territorium.getAnzahlKoerner(this.getReihe(), this
                        .getSpalte()) == 1
                && Territorium.getAnzahlKoerner(this.getReihe() - 2, this
                        .getSpalte()) > 0) {
            this.nimm(); // das eine Korn der aktuellen Mulde fressen
            this.linksUm();
            this.vor();
            this.vor();
            this.nimmAlle(); // gegenueberliegende Mulde leer fressen
            this.gotoKalahB();
            this.gibAlle(); // in eigenes Kalah ablegen
        }
    }

    /**
     * nach Kalah A laufen
     */
    protected void gotoKalahA() {
        this.laufeZuKachel(1, 0);
    }

    /**
     * nach Kalah B laufen
     */
    protected void gotoKalahB() {
        this.laufeZuKachel(1, Spielbrett.ANZAHL_MULDEN_PRO_SPIELER + 1);
    }
}
